import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox
import io
from typing import List, Dict, Any

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
BACKEND_HOST = "http://127.0.0.1:8000"


def fix_url(url):
    """
    If the provided URL does not start with http, assume it's a relative path.
    Remove any unwanted prefix (e.g., "app/static/") and prepend the public URL.
    """
    if url.startswith("http"):
        return url
    # If the URL starts with "app/static/", remove that part.
    prefix = "app/static/"
    if url.startswith(prefix):
        url = url[len(prefix) :]
    # Prepend the public URL
    return f"{BACKEND_HOST}/static/{url}"


def view_shop_frame(parent, switch_func, API_URL, token, shop_id):
    """
    CustomTkinter-based frame to display shop details and products.
    """
    frame = ctk.CTkFrame(parent, fg_color="transparent")

    # Fetch shop details and products from the backend
    headers = {"Authorization": f"Bearer {token}"}
    try:
        response = requests.get(f"{API_URL}/shops/get/{shop_id}", headers=headers)
        if response.status_code == 200:
            data = response.json()
            shop_data = data.get("shop", {})
            products = data.get("products", [])
        else:
            messagebox.showerror(
                "Error", f"Failed to fetch shop data. Status: {response.status_code}"
            )
            return frame
    except Exception as e:
        messagebox.showerror("Error", f"Request error: {e}")
        return frame

    # ------------- TOP LAYER (Shop Image and Details) -------------
    top_layer = ctk.CTkFrame(frame, fg_color="transparent")
    top_layer.place(relx=0, rely=0, relwidth=1, relheight=0.4)

    # Shop Image
    shop_image_label = ctk.CTkLabel(top_layer, text="")
    shop_image_label.place(relx=0.05, rely=0.1, relwidth=0.3, relheight=0.8)

    shop_image_url = shop_data.get("image_url")
    if shop_image_url:
        fixed_url = fix_url(shop_image_url)
        try:
            resp = requests.get(fixed_url)
            if resp.status_code == 200:
                pil_img = Image.open(io.BytesIO(resp.content)).resize((200, 200))
                tk_img = ImageTk.PhotoImage(pil_img)
                shop_image_label.configure(image=tk_img, text="")
                shop_image_label.image = tk_img
        except Exception as e:
            print(f"[DEBUG] Failed to load shop image: {e}")

    # Shop Details
    shop_details_frame = ctk.CTkFrame(top_layer, fg_color="transparent")
    shop_details_frame.place(relx=0.4, rely=0.1, relwidth=0.55, relheight=0.8)

    shop_name_label = ctk.CTkLabel(
        shop_details_frame,
        text=shop_data.get("name", "Shop Name"),
        font=("Helvetica", 20, "bold"),
    )
    shop_name_label.pack(pady=10)

    shop_description_label = ctk.CTkLabel(
        shop_details_frame,
        text=shop_data.get("description", "No description available."),
        font=("Helvetica", 14),
        wraplength=400,
        justify="left",
    )
    shop_description_label.pack(pady=5)

    shop_address_label = ctk.CTkLabel(
        shop_details_frame,
        text=f"Address: {shop_data.get('address', 'No address provided')}",
        font=("Helvetica", 12),
    )
    shop_address_label.pack(pady=5)

    # ------------- BOTTOM LAYER (Products) -------------
    bottom_layer = ctk.CTkFrame(frame, fg_color="transparent")
    bottom_layer.place(relx=0, rely=0.4, relwidth=1, relheight=0.6)

    products_label = ctk.CTkLabel(
        bottom_layer, text="Products", font=("Helvetica", 16, "bold")
    )
    products_label.pack(pady=10)

    products_frame = ctk.CTkScrollableFrame(
        bottom_layer, fg_color="transparent", width=750, height=300
    )
    products_frame.pack(fill="both", expand=True, padx=10, pady=5)

    def create_product_card(parent, product_data):
        card_width = 130
        card_height = 210
        card = ctk.CTkFrame(
            parent,
            corner_radius=5,
            fg_color="#2b2b2b",
            width=card_width,
            height=card_height,
        )
        card.pack_propagate(False)

        image_label = ctk.CTkLabel(card, text="")
        image_label.pack(pady=5)

        product_images = product_data.get("images", [])
        if product_images:
            img_url = product_images[0].get("image_url")
            if img_url:
                fixed_url = fix_url(img_url)
                try:
                    resp = requests.get(fixed_url)
                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize((70, 70))
                        tk_img = ImageTk.PhotoImage(pil_img)
                        image_label.configure(image=tk_img, text="")
                        image_label.image = tk_img
                except Exception as e:
                    print(f"[DEBUG] Product image error: {e}")

        name_text = product_data.get("name", "No Name")
        name_label = ctk.CTkLabel(
            card,
            text=name_text,
            font=("Helvetica", 11, "bold"),
            wraplength=120,
            anchor="center",
            justify="center",
        )
        name_label.pack(padx=5, pady=3)

        price_val = product_data.get("price", 0.0)
        price_label = ctk.CTkLabel(
            card,
            text=f"₫ {price_val:,.1f}",
            font=("Helvetica", 10, "bold"),
            text_color="#ff4242",
        )
        price_label.pack(side="bottom", pady=5)

        # Bind click event on card and its children.
        def on_click(event, product=product_data):
            switch_func("view_product", product)

        card.bind("<Button-1>", on_click)
        return card

    def display_products(products, container):
        for widget in container.winfo_children():
            widget.destroy()

        if not products:
            ctk.CTkLabel(container, text="No products found.").pack(pady=10)
            return

        grid_frame = ctk.CTkFrame(container, fg_color="transparent")
        grid_frame.pack(fill="both", expand=True)

        columns = 5  # 5 products per row
        for idx, product in enumerate(products):
            row = idx // columns
            col = idx % columns
            product_card = create_product_card(grid_frame, product)
            product_card.grid(row=row, column=col, padx=5, pady=5)

    # Display products
    display_products(products, products_frame)

    return frame


def view_product_frame(
    parent, switch_func, API_URL, token, product_data: Dict[Any, Any] = None
):
    """
    Create a professional product view frame similar to modern e-commerce sites.
    """
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    # Store token as a frame attribute so it can be accessed by add_to_cart
    frame.token = token

    def update_token(new_token):
        """Update the token when it changes"""
        nonlocal token
        token = new_token
        frame.token = new_token
        print(f"Product view token updated: {new_token}")  # Debug print

    # Add update_token method to frame
    frame.update_token = update_token

    # ------------- LEFT SIDEBAR -------------
    sidebar = ctk.CTkFrame(frame, fg_color=CARD_BG, width=60)
    sidebar.pack(side="left", fill="y")
    sidebar.pack_propagate(False)

    # Dashboard Icon
    dashboard_icon = ctk.CTkButton(
        sidebar,
        text="🏠",
        font=("Helvetica", 20),
        fg_color="transparent",
        text_color="white",
        hover_color="#3b3b3b",
        width=40,
        height=40,
        command=lambda: switch_func("dashboard"),
    )
    dashboard_icon.pack(pady=(20, 10))

    # User Icon
    user_icon = ctk.CTkButton(
        sidebar,
        text="👤",
        font=("Helvetica", 20),
        fg_color="transparent",
        text_color="white",
        hover_color="#3b3b3b",
        width=40,
        height=40,
        command=lambda: switch_func("user_details"),
    )
    user_icon.pack(pady=10)

    # Cart Icon
    cart_icon = ctk.CTkButton(
        sidebar,
        text="🛒",
        font=("Helvetica", 20),
        fg_color="transparent",
        text_color="white",
        hover_color="#3b3b3b",
        width=40,
        height=40,
        command=lambda: switch_func("user_orders"),
    )
    cart_icon.pack(pady=10)

    # ------------- MAIN CONTENT -------------
    main_content = ctk.CTkFrame(frame, fg_color="transparent")
    main_content.pack(side="left", fill="both", expand=True)

    def refresh_data(new_product_data: Dict[Any, Any]):
        """Update the frame with new product data"""
        nonlocal product_data
        if new_product_data:
            product_data = new_product_data
            update_product_display()

    def update_product_display():
        """Update all product information displays"""
        if not product_data:
            # Show placeholder state
            main_image_label.configure(text="Select a product to view details")
            product_name.configure(text="No product selected")
            price_label.configure(text="₫0")
            description_text.configure(state="normal")
            description_text.delete("1.0", "end")
            description_text.insert("1.0", "No description available.")
            description_text.configure(state="disabled")
            add_to_cart_btn.configure(state="disabled")
            return

        try:
            # Update product name
            product_name.configure(text=product_data.get("name", "Product Name"))

            # Update main image
            if product_images := product_data.get("images", []):
                if img_url := product_images[0].get("image_url"):
                    fixed_url = fix_url(img_url)
                    try:
                        resp = requests.get(fixed_url)
                        if resp.status_code == 200:
                            pil_img = Image.open(io.BytesIO(resp.content)).resize(
                                (400, 400)
                            )
                            tk_img = ImageTk.PhotoImage(pil_img)
                            main_image_label.configure(image=tk_img, text="")
                            main_image_label.image = tk_img

                            # Update thumbnails
                            for idx, img_data in enumerate(product_images[:4]):
                                if img_url := img_data.get("image_url"):
                                    fixed_url = fix_url(img_url)
                                    try:
                                        resp = requests.get(fixed_url)
                                        if resp.status_code == 200:
                                            pil_img = Image.open(
                                                io.BytesIO(resp.content)
                                            ).resize((80, 80))
                                            tk_img = ImageTk.PhotoImage(pil_img)
                                            thumb_labels[idx].configure(
                                                image=tk_img, text=""
                                            )
                                            thumb_labels[idx].image = tk_img
                                    except Exception as e:
                                        print(
                                            f"[DEBUG] Failed to load thumbnail {idx}: {e}"
                                        )
                                        thumb_labels[idx].configure(
                                            text=f"Thumb {idx + 1}"
                                        )
                    except Exception as e:
                        print(f"[DEBUG] Failed to load product image: {e}")
                        main_image_label.configure(text="Image not available")
            else:
                main_image_label.configure(text="No image available")
                for thumb_label in thumb_labels:
                    thumb_label.configure(text="No image")

            # Update price
            current_price = product_data.get("price", 0)
            price_label.configure(text=f"₫{current_price:,.0f}")

            # Update description
            description_text.configure(state="normal")
            description_text.delete("1.0", "end")
            description_text.insert(
                "1.0", product_data.get("description", "No description available.")
            )
            description_text.configure(state="disabled")

            # Enable add to cart button
            add_to_cart_btn.configure(state="normal")

        except Exception as e:
            messagebox.showerror("Error", f"Failed to update product display: {str(e)}")

    # Main container with two columns
    main_container = ctk.CTkFrame(main_content, fg_color="transparent")
    main_container.pack(fill="both", expand=True, padx=20, pady=20)

    # Left column for images
    left_column = ctk.CTkFrame(main_container, fg_color="transparent")
    left_column.pack(side="left", fill="both", expand=True)

    # Main image display
    main_image_frame = ctk.CTkFrame(left_column, fg_color=CARD_BG, corner_radius=10)
    main_image_frame.pack(fill="both", expand=True, padx=10, pady=10)

    main_image_label = ctk.CTkLabel(
        main_image_frame, text="Select a product to view details"
    )
    main_image_label.pack(padx=20, pady=20)

    # Thumbnail container
    thumbnail_container = ctk.CTkFrame(left_column, fg_color="transparent")
    thumbnail_container.pack(fill="x", pady=10)

    # Create thumbnail frames and labels
    thumb_labels = []
    for i in range(4):
        thumb_frame = ctk.CTkFrame(
            thumbnail_container, fg_color=CARD_BG, corner_radius=5
        )
        thumb_frame.pack(side="left", padx=5)
        thumb_label = ctk.CTkLabel(thumb_frame, text=f"Thumb {i + 1}")
        thumb_label.pack(padx=5, pady=5)
        thumb_labels.append(thumb_label)

    # Right column for product details
    right_column = ctk.CTkFrame(main_container, fg_color="transparent")
    right_column.pack(side="right", fill="both", expand=True, padx=20)

    # Product name
    product_name = ctk.CTkLabel(
        right_column,
        text="No product selected",
        font=("Helvetica", 24, "bold"),
        justify="left",
        wraplength=400,
    )
    product_name.pack(anchor="w", pady=(0, 10))

    # Price
    price_frame = ctk.CTkFrame(right_column, fg_color="transparent")
    price_frame.pack(fill="x", pady=10)

    price_label = ctk.CTkLabel(
        price_frame, text="₫0", font=("Helvetica", 24, "bold"), text_color="#ff4242"
    )
    price_label.pack(side="left")

    # Quantity selector
    quantity_frame = ctk.CTkFrame(right_column, fg_color="transparent")
    quantity_frame.pack(fill="x", pady=20)

    quantity_label = ctk.CTkLabel(
        quantity_frame, text="Quantity:", font=("Helvetica", 16, "bold")
    )
    quantity_label.pack(anchor="w", pady=(0, 10))

    quantity_selector = ctk.CTkFrame(quantity_frame, fg_color="#2b2b2b")
    quantity_selector.pack(side="left")

    quantity_var = ctk.IntVar(value=1)

    def update_quantity(delta):
        new_val = quantity_var.get() + delta
        if 1 <= new_val <= 99:
            quantity_var.set(new_val)

    minus_btn = ctk.CTkButton(
        quantity_selector, text="-", width=30, command=lambda: update_quantity(-1)
    )
    minus_btn.pack(side="left", padx=5, pady=5)

    quantity_entry = ctk.CTkEntry(
        quantity_selector, textvariable=quantity_var, width=50, justify="center"
    )
    quantity_entry.pack(side="left", padx=5, pady=5)

    plus_btn = ctk.CTkButton(
        quantity_selector, text="+", width=30, command=lambda: update_quantity(1)
    )
    plus_btn.pack(side="left", padx=5, pady=5)

    def add_to_cart():
        if not product_data:
            messagebox.showwarning("Warning", "No product selected")
            return

        try:
            # Validate quantity
            quantity = quantity_var.get()
            if not (1 <= quantity <= 99):
                messagebox.showwarning(
                    "Warning", "Please select a valid quantity (1-99)."
                )
                return

            # Get the current token from frame
            current_token = frame.token

            # Validate token
            if not current_token:
                messagebox.showerror("Error", "Please log in to add items to cart")
                return

            # Prepare data for API call
            headers = {"Authorization": f"Bearer {current_token}"}
            data = {
                "shop_id": product_data.get("shop_id"),
                "product_id": product_data.get("id"),
                "quantity": quantity,
                "price": product_data.get("price", 0),
            }

            # Debug print
            print(f"Token: {current_token}")
            print(f"Headers: {headers}")
            print(f"Data: {data}")

            # Make API call to add to cart
            response = requests.post(f"{API_URL}/cart/add", headers=headers, json=data)

            if response.status_code == 200:
                messagebox.showinfo("Success", "Product added to cart successfully!")
                # Optionally switch to cart view
                switch_func("user_orders")
            else:
                error_msg = response.json().get("detail", "Unknown error occurred")
                messagebox.showerror("Error", f"Failed to add to cart: {error_msg}")
                # Debug print
                print(f"Response status: {response.status_code}")
                print(f"Response body: {response.text}")

        except requests.RequestException as e:
            messagebox.showerror("Error", f"Network error: {str(e)}")
        except Exception as e:
            messagebox.showerror("Error", f"An unexpected error occurred: {str(e)}")
            # Debug print
            print(f"Exception: {str(e)}")

    # Add to cart button
    add_to_cart_btn = ctk.CTkButton(
        right_column,
        text="Add to Cart",
        font=("Helvetica", 16, "bold"),
        height=45,
        fg_color="#ff4242",
        hover_color="#ff6b6b",
        command=add_to_cart,
        state="disabled",  # Initially disabled
    )
    add_to_cart_btn.pack(fill="x", pady=20)

    # Product description
    description_label = ctk.CTkLabel(
        right_column, text="Product Description", font=("Helvetica", 16, "bold")
    )
    description_label.pack(anchor="w", pady=(20, 10))

    description_text = ctk.CTkTextbox(
        right_column, height=100, wrap="word", font=("Helvetica", 12)
    )
    description_text.pack(fill="x")
    description_text.insert("1.0", "No description available.")
    description_text.configure(state="disabled")

    # Set frame attributes
    frame.refresh_data = refresh_data

    # Initial display update if product data is available
    if product_data:
        update_product_display()

    return frame
